==> siuje-1.0.4.tgz: package/index.js <==
const { minute, hour, second } = require("./utime");
const { sorry, plaintext } = require("./uhttp");
const http = require("http");
const qstr = require("querystring");

const whenTimer = Symbol("wt"), pathMap = Symbol("@path"), swhen = Symbol("when");

const dummy = () => Promise.resolve(), forever = () => false;
let reuseDate = new Date();
const reuseNow = timestamp => (reuseDate.setTime(timestamp), reuseDate);
const minutestamp = ms => (reuseDate.setTime(ms + hour(8)), reuseDate.toISOString().replace("T", " ").slice(0, -8));

const paramRE = pattern => {
    const arrparam = pattern.match(new RegExp("/:([^/]*)", "g")).map( r=>r.slice(2) );
    const newregstr = pattern.replace( new RegExp("/:([^/]*)", "g"), "/(.*)" );
    return (origin, result) => {
        const res = new RegExp(newregstr).exec( origin );
        if( res == null ) return false;
        return !!arrparam.reduce( (r, key, index) => (r[key]=res[index+1], r), result );
    }
}

const siuje = {
    log( logLevel, message ) {
        if( message == undefined ) [ logLevel, message ] = [ "INFO", logLevel ];
        console.info( `[${logLevel}][${reuseNow(Date.now()).toISOString()}] ${message}` );
    },

    when(shouldwork) {
        const root = this;
        root[swhen] = root[swhen] || [];
        let task = { shouldwork, action: dummy };
        const round = async () => {
            root[whenTimer] = setTimeout(round, minute(1));
            const now = Date.now();
            await Promise.all(root[swhen].map(task => {
                if (typeof task.shouldwork == "string") {
                    if (minutestamp(now).includes(task.shouldwork))
                        return task.action();
                    else return dummy();
                }
                else if (task.shouldwork(reuseNow(now)))
                    return task.action();
                else return dummy();
            }).filter(n => (n||{}).then));
        }
        return {
            work(action, initial) {
                task.action = action;
                if (root[swhen].length == 0)
                    root[whenTimer] = setTimeout(round, minute(1));
                root[swhen].push(task);
                if (initial) initial();
            }
        }
    },

    every(seconds) {
        let task = { timer: 0, timeval: seconds, finish: forever, action: dummy, };
        const dotask = async () => {
            let next = Date.now() + second(task.timeval);
            if (!task.finish()) {
                await task.action();
                task.timer = setTimeout(dotask, next - Date.now());
            }
        }
        return {
            repeat(action, initial) {
                task.action = action;
                if (initial) initial();
                task.timer = setTimeout(dotask, second(task.timeval));
                return {
                    until(taskend) {
                        task.finish = taskend;
                    }
                }
            }
        }
    },

    meet(request) {
        const root = this;
        this[pathMap] = this[pathMap] || new Map();
        let tools = [];
        const apply = handler => {
            while (tools.length > 0) {
                const tool = tools.pop();
                const enhanced = handler;
                handler = (req, res) => tool(req, res)(enhanced).catch( err => (res(500), root.log("ERRR", `request fatal: ${err.stack}`),  sorry) );
            }
            return handler;
        }
        const fullfilled = Symbol("fullfilled");
        const simplepath = false == new RegExp("/:([^/]*)", "g").test(request);
        const regexparam = simplepath ? null : paramRE(request);
        const proxy = {
            use(tool) {
                if( this[fullfilled] ) 
                throw new Error(`call 'use' in serving request[${request}]`);
                if( Array.isArray(tool) ) tools = tools.concat(tool);
		else tools.push(tool);
                return this;
            },

            serve(action) {
                if( this[fullfilled] ) 
                    throw new Error(`call 'serve' in serving request[${request}]`);
                this[fullfilled] = true;
                if( !simplepath ){
                    const pathrules = root[pathMap][siuje] || [];
                    pathrules.push( {check: regexparam, handler: apply(action)} );
                    root[pathMap][siuje] = pathrules;
                }
                else root[pathMap][request] = apply(action);
            }
        }
        return proxy;
    },

    work( config ){
        const server = http.createServer( (req, res) => {
            siuje.log(`start serving [${req.method}] request [${req.url}]`);

            let url = new URL( req.url, `http://localhost` );
            let ctx = {
                status: 404,
                header: {},
                format: "text"
            };
            let output = body => {
                let respbody = "", ctype = "";
                if( ctx.format == "json" )
                    respbody = JSON.stringify( body ), ctype = "application/json; charset=utf-8";
                else if( ctx.format == "query" )
                    respbody = qstr.stringify( body );
                else respbody = String(body), ctype = "text/plain";
                return res.writeHead( ctx.status, ctype ? {...ctx.header, "Content-Type": ctype} : ctx.header ).end( respbody );
            }
            const onfail = reason => {
		    siuje.log( "ERRR", `request reject: ${reason.stack}` );
		    res.writeHead( 500 ).end( reason.message );
	    }
            const redirect_output = new_output => output = new_output;
            const syn = v => {
                const type = typeof v;
                if( type == "string" )
                    ctx.format = v;
                else if( type == "number" )
                    ctx.status = v;
                else if( type instanceof Error )
		    onfail( reason );
                else if( type == "object" )
                    ctx.header = Object.assign( ctx.header, v );
                else if( type == "function" )
                    return v( res, redirect_output  );
                else if( type == "undefined" )
                    return [ctx.status, ctx.header, ctx.format];
                return syn;
            }
            try{
                if( this[pathMap][url.pathname] ){
                    ctx.status = 200;
                    this[pathMap][url.pathname]( req, syn ).then( output );
                }
                else if( this[pathMap][siuje] ) {
                    const param = {};
                    const worker = this[pathMap][siuje].find( w=>w.check(url.pathname, param) );
                    if( worker ){
                        req.param = param;
                        ctx.status = 200;
                        worker.handler( req, syn ).then( output );
                    }
                    else sorry.then( output );
                }
		else sorry.then( output );
	    }catch( err ) { onfail( err ); }
        } );
        server.listen( config, error => siuje.log(error ? `fatal at listen: ${error.stack}` : `server started at ${config.host || "localhost"}:${config.port || 80} with pid ${process.pid}...`) );

        const exitbysig = signal => {
            process.on( signal, () => server.close( () => {
                siuje.log( `halt by signal[${signal}]` );
                if( config.halt ) process.exit( -1 );
            } ) );
            return exitbysig;
	}
        exitbysig( "SIGABRT" )( "SIGTERM" )( "SIGINT" );
        process.on( "SIGHUP", () => siuje.log("ignore by signal[HUP]") );

        server.on( "error", error => siuje.log(`server get error event: ${error.stack}`) );
        process.on( "unhandledRejection", error => siuje.log(`uncatch exception: ${error.stack}`));
        process.on( "uncaughtException", error => siuje.log(`unhandle rejection: ${error.stack}`));
    }
}

module.exports = siuje;

==> siuje-1.0.5.tgz: package/index.js <==
const { minute, hour, second } = require("./utime");
const { sorry, plaintext } = require("./uhttp");
const http = require("http");
const qstr = require("querystring");

const whenTimer = Symbol("wt"), pathMap = Symbol("@path"), swhen = Symbol("when");

const dummy = () => Promise.resolve(), forever = () => false;
let reuseDate = new Date();
const reuseNow = timestamp => (reuseDate.setTime(timestamp), reuseDate);
const minutestamp = ms => (reuseDate.setTime(ms + hour(8)), reuseDate.toISOString().replace("T", " ").slice(0, -8));

const paramRE = pattern => {
    const arrparam = pattern.match(new RegExp("/:([^/]*)", "g")).map( r=>r.slice(2) );
    const newregstr = pattern.replace( new RegExp("/:([^/]*)", "g"), "/(.*)" );
    return (origin, result) => {
        const res = new RegExp(newregstr).exec( origin );
        if( res == null ) return false;
        return !!arrparam.reduce( (r, key, index) => (r[key]=res[index+1], r), result );
    }
}

const siuje = {
    log( logLevel, message ) {
        if( message == undefined ) [ logLevel, message ] = [ "INFO", logLevel ];
        console.info( `[${logLevel}][${reuseNow(Date.now()).toISOString()}] ${message}` );
    },

    when(shouldwork) {
        const root = this;
        root[swhen] = root[swhen] || [];
        let task = { shouldwork, action: dummy };
        const round = async () => {
            root[whenTimer] = setTimeout(round, minute(1));
            const now = Date.now();
            await Promise.all(root[swhen].map(task => {
                if (typeof task.shouldwork == "string") {
                    if (minutestamp(now).includes(task.shouldwork))
                        return task.action();
                    else return dummy();
                }
                else if (task.shouldwork(reuseNow(now)))
                    return task.action();
                else return dummy();
            }).filter(n => (n||{}).then));
        }
        return {
            work(action, initial) {
                task.action = action;
                if (root[swhen].length == 0)
                    root[whenTimer] = setTimeout(round, minute(1));
                root[swhen].push(task);
                if (initial) initial();
            }
        }
    },

    every(seconds) {
        let task = { timer: 0, timeval: seconds, finish: forever, action: dummy, };
        const dotask = async () => {
            let next = Date.now() + second(task.timeval);
            if (!task.finish()) {
                await task.action();
                task.timer = setTimeout(dotask, next - Date.now());
            }
        }
        return {
            repeat(action, initial) {
                task.action = action;
                if (initial) initial();
                task.timer = setTimeout(dotask, second(task.timeval));
                return {
                    until(taskend) {
                        task.finish = taskend;
                    }
                }
            }
        }
    },

    meet(request) {
        const root = this;
        this[pathMap] = this[pathMap] || new Map();
        let tools = [];
        const apply = handler => {
            while (tools.length > 0) {
                const tool = tools.pop();
                const enhanced = handler;
                handler = (req, res) => tool(req, res)(enhanced).catch( err => (res(500), root.log("ERRR", `request fatal: ${err.stack}`),  sorry) );
            }
            return handler;
        }
        const fullfilled = Symbol("fullfilled");
        const simplepath = false == new RegExp("/:([^/]*)", "g").test(request);
        const regexparam = simplepath ? null : paramRE(request);
        const proxy = {
            use(tool) {
                if( this[fullfilled] ) 
                throw new Error(`call 'use' in serving request[${request}]`);
                if( Array.isArray(tool) ) tools = tools.concat(tool);
		else tools.push(tool);
                return this;
            },

            serve(action) {
                if( this[fullfilled] ) 
                    throw new Error(`call 'serve' in serving request[${request}]`);
                this[fullfilled] = true;
                if( !simplepath ){
                    const pathrules = root[pathMap][siuje] || [];
                    pathrules.push( {check: regexparam, handler: apply(action)} );
                    root[pathMap][siuje] = pathrules;
                }
                else root[pathMap][request] = apply(action);
            }
        }
        return proxy;
    },

    work( config ){
        const server = http.createServer( (req, res) => {
            siuje.log(`start serving [${req.method}] request [${req.url}]`);

            let url = new URL( req.url, `http://localhost` );
            let ctx = {
                status: 404,
                header: {},
                format: null
            };
            let output = body => {
                let respbody = "", ctype = "";
                if( ctx.format == "json" )
                    respbody = JSON.stringify( body ), ctype = "application/json; charset=utf-8";
                else if( ctx.format == "query" )
                    respbody = qstr.stringify( body );
                else respbody = String(body), ctype = ctx.format ?? "text/plain";
                return res.writeHead( ctx.status, ctype ? {...ctx.header, "Content-Type": ctype} : ctx.header ).end( respbody );
            }
            const onfail = reason => {
		    siuje.log( "ERRR", `request reject: ${reason.stack}` );
		    res.writeHead( 500 ).end( reason.message );
	    }
            const redirect_output = new_output => output = new_output;
            const syn = v => {
                const type = typeof v;
                if( type == "string" )
                    ctx.format = v;
                else if( type == "number" )
                    ctx.status = v;
                else if( type instanceof Error )
		    onfail( reason );
                else if( type == "object" )
                    ctx.header = Object.assign( ctx.header, v );
                else if( type == "function" )
                    return v( res, redirect_output  );
                else if( type == "undefined" )
                    return [ctx.status, ctx.header, ctx.format];
                return syn;
            }
            try{
                if( this[pathMap][url.pathname] ){
                    ctx.status = 200;
                    this[pathMap][url.pathname]( req, syn ).then( output );
                }
                else if( this[pathMap][siuje] ) {
                    const param = {};
                    const worker = this[pathMap][siuje].find( w=>w.check(url.pathname, param) );
                    if( worker ){
                        req.param = param;
                        ctx.status = 200;
                        worker.handler( req, syn ).then( output );
                    }
                    else sorry.then( output );
                }
		else sorry.then( output );
	    }catch( err ) { onfail( err ); }
        } );
        server.listen( config, error => siuje.log(error ? `fatal at listen: ${error.stack}` : `server started at ${config.host || "localhost"}:${config.port || 80} with pid ${process.pid}...`) );

        const exitbysig = signal => {
            process.on( signal, () => server.close( () => {
                siuje.log( `halt by signal[${signal}]` );
                if( config.halt ) process.exit( -1 );
            } ) );
            return exitbysig;
	}
        exitbysig( "SIGABRT" )( "SIGTERM" )( "SIGINT" );
        process.on( "SIGHUP", () => siuje.log("ignore by signal[HUP]") );

        server.on( "error", error => siuje.log(`server get error event: ${error.stack}`) );
        process.on( "unhandledRejection", error => siuje.log(`uncatch exception: ${error.stack}`));
        process.on( "uncaughtException", error => siuje.log(`unhandle rejection: ${error.stack}`));
    }
}

module.exports = siuje;
